@@ -6,5 +6,5 @@ from rest_framework_swagger.views import SwaggerResourcesView, SwaggerApiView, S
 urlpatterns = patterns('',
     url(r'^$', SwaggerUIView.as_view(), name="django.swagger.base.view"),
     url(r'^api-docs/$', SwaggerResourcesView.as_view(), name="django.swagger.resources.view"),
-    url(r'^api-docs/(?P<path>.*)/$', SwaggerApiView.as_view(), name='django.swagger.api.view'),
+    url(r'^api-docs/(?P<path>.*)/?$', SwaggerApiView.as_view(), name='django.swagger.api.view'),
 )
